####################################################################################
# R-Code presented at Frankfurter Stochastik-Tage 2006, 16.03.06
# author: P. Ruckedeschel
# title: S4 classes for distributions and extensions
####################################################################################


#### local settings:

#PATH (to be modified locally!)
path <- "D:/Dissertation/Vortrge/Frankfurt/Peter/"


##Preparation: loading packages distr, distrEx 
#    (first one is loaded automatically when second is loaded)
require("distrEx")



####################################################################################
# some worked out functionals
####################################################################################

source(paste(path,"functionals.R", sep = ""))

## sourcing in new functionals to be included in the next release of distrEx (v1.7)


### these include:
#
#   var() to be called just as E()
#   sd() (for univariate distributions)
#   median() (for univariate distributions)
#   mad() (for univariate distributions / not standardized for normal as mad(data) ! )
#   IQR() (for univariate distributions / not standardized for normal as IQR(data) ! )
### intentionally mask var in order to have an additional ... argument


####################################################################################
# motivational example
####################################################################################


N <- Norm(mean = 2, sd = 1.3)
P <- Pois(lambda = 1.2)
Z <- 2 * N + 3 + P # exact transformation

### examining what N, P, Z are:
plot(Z)
p(Z)(0.4)
q(Z)(0.3)
r(Z)(10)

## something weird
Znew <- sin(abs(Z)) # by simulations
plot(Znew)
p(Znew)(0.2)


####################################################################################
# example expectation operator
####################################################################################
require("distrEx")

D1 <- Norm(mean=2)
m1 <- E(D1)  # = 2
E(D1, function(x){ x^2 }) # E(D1^2)
# integrand with additional argument:
E(D1, function(x, m1){(x - m1)^2}, m1 = m1) # '$\Var$'
# same way
sd(D1);median(D1);mad(D1);IQR(D1)

## now same code but for Poisson:

D1 <- Pois(lambda=3)
m1 <- E(D1) # = 3
E(D1, function(x){ x^2 })
E(D1, function(x, m1){(x - m1)^2}, m1 = m1)
sd(D1);median(D1);mad(D1);IQR(D1)


## for later purposes: general standardization routine

make01 <- function(x){
    if (!is(x, "UnivariateDistribution"))
        stop("This function is for distribution objects.")
    return((x-E(x))/sd(x))
    }

##########################################################################################
#
# Illustration 1: illustration of CLT due to M.Kohl
#
##########################################################################################

source(paste(path,"illustCLT.R", sep = ""))

### loads methods for illustrateCLT
## illustrateCLT produces plots for the subseqent laws of T_n
## syntax: illustrateCLT(Distr, len, sleep = 0) where 
##         Distr ^= distribution of X_i, 
##           len ^= # summands at which to stop
##         sleep ^= duration of the pause between the different plots


# some examples
distroptions("DefaultNrFFTGridPointsExponent", 12)
illustrateCLT(Distr = Unif(), len = 20)
illustrateCLT(Distr = Exp(), len = 20)
illustrateCLT(Distr = Chisq(), len = 20)
illustrateCLT(Distr = Td(df = 5), len = 20)
illustrateCLT(Distr = Beta(), len = 20)
distroptions("DefaultNrFFTGridPointsExponent", 13)
illustrateCLT(Distr = Lnorm(), len = 20)

distroptions("DefaultNrFFTGridPointsExponent", 12)
illustrateCLT(Distr = Pois(lambda = 2), len = 20)
illustrateCLT(Distr = Binom(size = 5), len = 20)
illustrateCLT(Distr = Nbinom(size = 5), len = 20)

##########################################################################################
#
# Illustration 2:  ML / MD -estimators
#
##########################################################################################

source(paste(path,"estim.R", sep = ""))
## defines (preliminarily) a new class "nParamFamily" with corresponding generating function
## slots/initialization arguments
##        name (string; name of the parametricmodel),
##        distribution (Distribution; the [parametrized] distribution of the [ideal] model
##        distrSymm     (symmetry class; not discussed here; serves to use symmetry information) 
##        modifparameter (function with argument theta) the mapping theta -> P_theta
##        main           (numeric; value of main parameter)
##        nuisance       (numeric; value of (parametric) nuisance parameter)
##        trafo          (matrix; if we are interested in a transformed parameter T(theta);
##                                D=T'(theta))
##        props          (string; list of properties/calculated terms gathered while building up this object)
###########################################################################################

##examples
NormalLocation <- nParamFamily("Normal location", modifparameter = function(theta) Norm(mean=theta))
NormalLocScale <- nParamFamily("Normal location and scale", 
                       modifparameter = function(theta) Norm(mean=theta[1],sd=abs(theta[2])), main=c(0,1))
                       
Poisson <- nParamFamily("Poisson", distribution = Pois(), modifparameter = function(theta) Pois(lambda=theta))

## for later purposes: generate some data according to these models
datNormal  <- rnorm(100, sd = 3,mean = 4)                       
datPoisson <- rpois(100, lambda = 2)                       

### examples of estimator evaluations
## MDE(ParaModel, data, distance,...) and MLE(ParaModel, data,...) are interfaces to fitParam
# --- MDE uses a distance, MLE the negative log-likelihood as criterium to be optimized in fitParam
#
# sketchy:
# fitParam <- function(model, data0, criterium ....)
#   { #define a function in theta to be optimized:
#     ftoOptimize <- function(theta)
#        {Ptheta <- modifparameter(model)(theta) # generate Ptheta
#         criterium(Ptheta,data0)
#         }
#     #in one dimension use  "optimize", in higher "optim"
#       {if(dimension(param(model)) == 1)
#           theta <- optimize(f = ftoOptimize, interval = searchinterval0, ...)$minimum
#        else 
#           theta <- optim(par = initval0, f = ftoOptimize, ...)$par
#       }
#   return(theta) 
#   }
#########
## criterium takes to arguments, a distribution /distr/ and the data /dat/ and 
## qualifies the fit of /distr/ for /dat/  [the lower the value the better] 

MDE(NormalLocation,datNormal,searchint=c(min(datNormal),max(datNormal)))
MLE(NormalLocation,datNormal,searchint=c(min(datNormal),max(datNormal)))
   
(th0 <- MLE(NormalLocScale,datNormal))
MDE(NormalLocScale,datNormal, init=th0)

MLE(Poisson,datPoisson,searchint=c(0,max(datPoisson)))
MDE(Poisson,datPoisson,searchint=c(0,max(datPoisson)))
   

##########################################################################################
#
# Illustration 3:  Deconvolution / posterior mode / -mean
#
##########################################################################################

source(paste(path,"PrognCondDistribution.R", sep = ""))

###
#defines a class PrognCondDistribution and a corresponding generating function
#PrognCondDistribution <- function(Regr = Norm(), Error = Norm()), where 
# /Regr/ and /Error/ are the distributions of X and Error resp., both defaulting to N(0,1)


## by M. Kohl
## slightly modified by P.R. [for more recent versions of our packages]
IntegrateTruncQuantile <- 0 # zur Integration

D1 <- PrognCondDistribution(Error = Td(1))
D2 <- PrognCondDistribution(Error = Td(3))
D3 <- PrognCondDistribution(Error = ConvexContamination(Norm(), Norm(4,1), size=0.1))
D4 <- PrognCondDistribution(Error = ConvexContamination(Norm(), Norm(0,9), size=0.1))
D5 <- PrognCondDistribution(Error = ConvexContamination(Norm(), Norm(0,9), size=0.2))

y <- seq(from = 0, to = 8, length = 100)
## conditional expectation
f <- function(y, e1){ E(e1, function(x){x[1]}, y) }

## takes some time ...
system.time(erg1 <- sapply(y, f, D1)) 
system.time(erg2 <- sapply(y, f, D2)) 
system.time(erg3 <- sapply(y, f, D3)) 
system.time(erg4 <- sapply(y, f, D4)) 
system.time(erg5 <- sapply(y, f, D5)) 

## posterior mode
post.mod <- function(cond, e1) {
    optimize(f = d(e1), interval = c(q(e1)(1e-3, cond), q(e1)(1-1e-3, cond)), 
        tol = .Machine$double.eps^0.25, maximum = TRUE, cond = cond)$maximum
}

D0 <- PrognCondDistribution()

## takes some time ...
system.time(perg0 <- sapply(y, post.mod, D0)) 
system.time(perg1 <- sapply(y, post.mod, D1)) 
system.time(perg2 <- sapply(y, post.mod, D2)) 
system.time(perg3 <- sapply(y, post.mod, D3)) 
system.time(perg4 <- sapply(y, post.mod, D4)) 
system.time(perg5 <- sapply(y, post.mod, D5)) 

## plot
plot(y, 0.5*y, type = "l")
lines(y, erg1, col = "red")
lines(y, erg2, col = "blue")
lines(y, erg3, col = "orange")
lines(y, erg4, col = "darkgreen")
lines(y, erg5, col = "darkred")
title("Conditional Expectation")

windows()
plot(y, perg0, type = "l", ylim = c(-0.5, 4))
lines(y, perg1, col = "red")
lines(y, perg2, col = "blue")
lines(y, perg3, col = "orange")
lines(y, perg4, col = "darkgreen")
lines(y, perg5, col = "darkred")
title("Posterior Mode")
